@@ -1,20 +1,24 @@
 # coding: utf-8
 
+from __future__ import print_function, absolute_import, division, unicode_literals
+
 _package_data = dict(
     full_package_name='ruamel.std.pathlib',
-    version_info=(0, 9, 2),
-    __version__='0.9.2',
-    version_timestamp='2022-02-19 08:36:38',
+    version_info=(0, 13, 0),
+    __version__='0.13.0',
+    version_timestamp='2024-03-03 07:56:23',
     author='Anthon van der Neut',
     author_email='a.van.der.neut@ruamel.eu',
     since=2013,
-    description='improvements over the standard pathlib module',
+    description='improvements over the standard pathlib module and pathlib2 package',
     entry_points=None,
+    extras_require={':python_version<="3.4"': ['pathlib2']},
     license='MIT License',
-    tox=dict(env='*',),
-    python_requires='>=3',  # 3.7
-    print_allowed=True,
+    # universal=True,
+    python_requires='>=3',
+    tox=dict(env='3', deps=['orjson', 'ujson']),
 )
+# pathlib2 for 3.4 -> expanduser added
 
 version_info = _package_data['version_info']
 __version__ = _package_data['__version__']
@@ -23,54 +27,27 @@ __version__ = _package_data['__version__']
 
 import os  # NOQA
 import sys  # NOQA
+import time  # NOQA
 import inspect  # NOQA
-import pathlib  # NOQA
 
-from pathlib import *  # NOQA
+if sys.version_info < (3, 5):
+    # 3.4 e.g has no expanduser()
+    from pathlib2 import *  # NOQA
 
-from typing import Any, Union, Optional, Dict, Generator, Tuple, List  # NOQA
-
-
-# https://bugs.python.org/issue24132#msg401946
-# without this you cannot subclass or assign data attributes (methods always worked fine)
-class Path(type(pathlib.Path())):  # type: ignore
-    def utime_ns(self, atime_ns: int, mtime_ns: int) -> None:
-        os.utime(self, ns=(atime_ns, mtime_ns), follow_symlinks=False)
-
-    def read_bytes_atime_preserve(self, *arg: Any, **kw: Any) -> bytes:
-        stat = os.stat(self)
-        res: bytes = self.read_bytes(*arg, **kw)
-        os.utime(self, ns=(stat.st_atime_ns, stat.st_mtime_ns), follow_symlinks=False)
-        return res
-
-    def read_text_atime_preserve(self, *arg: Any, **kw: Any) -> str:
-        stat = os.stat(self)
-        res: str = self.read_text(*arg, **kw)
-        os.utime(self, ns=(stat.st_atime_ns, stat.st_mtime_ns), follow_symlinks=False)
-        return res
-
-
-if sys.version_info < (3, 9):
-
-    def _is_relative_to(self, other):
-        try:
-            self.relative_to(other)
-            return True
-        except ValueError:
-            return False
-
-    Path.is_relative_to = _is_relative_to
+    FileNotFoundError = IOError
+else:
+    from pathlib import *  # NOQA
 
 
 class DirStack(object):
-    def __init__(self) -> None:
-        self._dir_stack: list[str] = []
+    def __init__(self):
+        self._dir_stack = []
 
-    def pushd(self, d: Union[str, Path]) -> None:
+    def pushd(self, d):
         self._dir_stack.insert(0, os.getcwd())
         os.chdir(str(d))
 
-    def popd(self) -> None:
+    def popd(self):
         os.chdir(self._dir_stack.pop())
 
 
@@ -78,40 +55,50 @@ class DirStack(object):
 _dir_stack = DirStack()
 
 
-def pushd(d: Union[str, Path]) -> None:
+def pushd(d):
     _dir_stack.pushd(d)
 
 
-def popd() -> None:
+def popd():
     _dir_stack.popd()
 
 
+# pathlib 0.8 was not compatible with pathlib in python 3.4,
+# but 0.97 is
+# should check here but there is no version number in pathlib ...
+# switched to pathlib2 which aims at compatibility
+#
+#     @property
+#     def root_stem(self):
+#         """The final path component, minus suffices."""
+#         return self.name.split('.', 1)[0]
+
+
 # just for some sanity
-pathlib.Path.remove = pathlib.Path.unlink  # type: ignore
+Path.remove = Path.unlink
 
 
-def _rmtree(self: Path) -> None:
+def _rmtree(self):
     import shutil
 
     assert self.is_dir()
     shutil.rmtree(str(self))
 
 
-Path.rmtree = _rmtree  # type: ignore
+Path.rmtree = _rmtree
 
 
-def _chdir(self: Path) -> None:
+def _chdir(self):
     assert self.is_dir()
     os.chdir(str(self))
 
 
-Path.chdir = _chdir  # type: ignore
+Path.chdir = _chdir
 
 
-def _hash(self: Path, typ: Optional[str] = None, size: int = -1) -> Any:
+def _hash(self, typ=None, size=-1):
     """hash of the contents, typ can be any of the hashlib.new() acceptable strings
-    if size is provided and non-negative only read that amount of bytes from the start
-    of the file
+    if size is provided and positive only read that amount of bytes from the start of the file
     """
     import hashlib
 
@@ -119,39 +106,49 @@ def _hash(self: Path, typ: Optional[str] = None, size: int = -1) -> Any:
         typ = 'sha224'
     with self.open(mode='rb') as f:
         data = f.read(size)
-    h = hashlib.new(typ, data=data)
+    try:
+        # calling hashlib.sha224() is faster, but does it take data argument?
+        h = hashlib.new(typ, data=data)
+    except TypeError:
+        # 2.7 didn't have data argument to new
+        h = hashlib.new(typ)
+        h.update(data)
     return h
 
 
-pathlib.Path.hash = _hash  # type: ignore
+Path.hash = _hash
 
 
-def _copy(self: Path, target: Union[str, Path]) -> None:
+def _copy(self, target):
     import shutil
 
     assert self.is_file()
     shutil.copy(str(self), str(target))
 
 
-pathlib.Path.copy = _copy  # type: ignore
+Path.copy = _copy
 
 
 # this is the times parameter of os.utime, taking int/float seconds (and not nanoseconds)
-if sys.version_info < (3, 6):
-
-    def _utime(self: Path, atime: int, mtime: int) -> None:
-        os.utime(str(self), (atime, mtime), follow_symlinks=False)
+def _utime(self, atime, mtime):
+    os.utime(str(self), (atime, mtime), follow_symlinks=False)
 
 
-else:
-
-    def _utime(self: Path, atime: int, mtime: int) -> None:
-        os.utime(self, (atime, mtime), follow_symlinks=False)
+Path.utime = _utime
 
 
-pathlib.Path.utime = _utime  # type: ignore
+def _stabilise(self, duration=5, recheck=0.2, msg=None):
+    """wait if modified in the last duration seconds"""
+    while (self.stat().st_mtime + duration) > time.time():
+        if msg:
+            print(msg)
+            msg = None
+        time.sleep(recheck)
 
 
+Path.stabilise = _stabilise
+Path.stabilize = _stabilise
+
 # #
 # # - If the check level is set at 1, any calls that are issued with strings that
 # #     could be changed are logged (to the console), those are the positions where
@@ -184,12 +181,12 @@ class PathLibConversionHelper(object):
     and incorporate
     """
 
-    def __init__(self, check: int = 0) -> None:
+    def __init__(self, check=0):
         class Container(object):
             pass
 
         self._check = check
-        self._usage: Dict[Any, list[int]] = {}
+        self._usage = {}
         path = self.path = Container()
         for m in dir(self):
             if not m.startswith('_path_'):
@@ -197,7 +194,7 @@ class PathLibConversionHelper(object):
             attr = m.split('_', 2)[2]
             setattr(path, attr, getattr(self, m))
 
-    def add_usage(self, vars: Any, comment: str) -> None:
+    def add_usage(self, vars, comment):
         if isinstance(vars, (list, tuple)):
             vt = tuple([isinstance(x, Path) for x in vars])
         else:
@@ -206,12 +203,12 @@ class PathLibConversionHelper(object):
             return
         caller = inspect.stack()[2]
         t = caller[1], caller[2], comment, vt
-        count = self._usage.setdefault(t, [0])
+        count = self._usage.setdefault(t, [0,])
         count[0] += 1
         if self._check > 1 and count[0] == 1:
             print('{2} [{0}:{1} / Path {3}]'.format(*t))
 
-    def dump(self, stream: Any = None, show_all: bool = False) -> None:
+    def dump(self, stream=None, show_all=False):
         """dump unique invocations of methods
 
         unless show_all is True, invocations that uniquely use Path arguments
@@ -227,7 +224,7 @@ class PathLibConversionHelper(object):
                     )
                 )
 
-    def rename(self, old_name: Union[str, Path], new_name: Union[str, Path]) -> None:
+    def rename(self, old_name, new_name):
         """os.rename replacement that can handle Path argument"""
         self.add_usage([old_name, new_name], 'update .rename')
         if isinstance(old_name, Path):
@@ -239,13 +236,11 @@ class PathLibConversionHelper(object):
     # os.getcwd
     # os.path.expanduser
 
-    def walk(
-        self, top: Union[str, Path], *args: Any, **kw: Any
-    ) -> Generator[Tuple[Path, List[str], List[str]], None, None]:
+    def walk(self, top, *args, **kw):
         for r, d, f in os.walk(str(top), *args, **kw):
             yield Path(r), d, f
 
-    def _path_join(self, base: Union[str, Path], *args: str) -> Union[str, Path]:
+    def _path_join(self, base, *args):
         self.add_usage([base] + list(args), 'update .path.join to use "/"')
         # os.path.join
         if isinstance(base, Path):
@@ -253,49 +248,49 @@ class PathLibConversionHelper(object):
         else:
             return os.path.join(base, *args)
 
-    def _path_exists(self, path: Union[str, Path]) -> bool:
+    def _path_exists(self, path):
         self.add_usage(path, 'update .exists to use Path.exists()')
         if isinstance(path, Path):
             return path.exists()
         else:
             return os.path.exists(path)
 
-    def _path_dirname(self, file_name: Union[str, Path]) -> Union[str, Path]:
+    def _path_dirname(self, file_name):
         self.add_usage(file_name, 'update .path.dirname to use Path.parent')
         if isinstance(file_name, Path):
             return file_name.parent
         else:
             return os.path.dirname(file_name)
 
-    def _path_getmtime(self, file_name: Union[str, Path]) -> float:
-        self.add_usage(file_name, 'update .path.getmtime to use Path.stat().st_mtime')
+    def _path_getmtime(self, file_name):
+        self.add_usage(file_name, 'update .path.getmtime to use Path.parent')
         if isinstance(file_name, Path):
             return file_name.stat().st_mtime
         else:
             return os.path.getmtime(file_name)
 
-    def _path_expanduser(self, path: Union[str, Path]) -> Union[str, Path]:
+    def _path_expanduser(self, path):
         self.add_usage(path, 'update .path.expanduser to use Path.expanduser')
         if isinstance(path, Path):
             return path.expanduser()
         else:
             return os.path.expanduser(path)
 
-    def _path_basename(self, path: Union[str, Path]) -> str:
+    def _path_basename(self, path):
         self.add_usage(path, 'update .path.basename to use Path.basename')
         if isinstance(path, Path):
             return path.name
         else:
             return os.path.basename(path)
 
-    def _path_isdir(self, path: Union[str, Path]) -> bool:
+    def _path_isdir(self, path):
         self.add_usage(path, 'update .path.isdir to use Path.isdir')
         if isinstance(path, Path):
             return path.is_dir()
         else:
             return os.path.isdir(path)
 
-    def _path_splitext(self, path: Union[str, Path]) -> Any:
+    def _path_splitext(self, path):
         self.add_usage(
             path, 'update .path.splitext to use Path.parent / Path.stem + Path.suffix'
         )
@@ -304,64 +299,64 @@ class PathLibConversionHelper(object):
         else:
             return os.path.splitext(path)
 
-    def chdir(self, path: Union[str, Path]) -> Any:
+    def chdir(self, path):
         """replaces: os.chdir()"""
         self.add_usage(path, 'update .chdir to use Path.chdir() or os.chdir(str(Path))')
         if isinstance(path, Path):
-            return path.chdir()  # type: ignore
+            return path.chdir()
         else:
             return os.chdir(path)
 
     # os.mkdir
 
-    def rmdir(self, path: Union[str, Path]) -> bool:
+    def rmdir(self, path):
         """replaces: os.rmdir()"""
         self.add_usage(path, 'update .rmdir to use Path.rmdir() or os.rmdir(str(Path))')
         if isinstance(path, Path):
-            return path.rmdir()  # type: ignore
+            return path.rmdir()
         else:
-            return os.rmdir(path)  # type: ignore
+            return os.rmdir(path)
 
-    def makedirs(self, path: Union[str, Path], *args: Any, **kw: Any) -> None:
+    def makedirs(self, path, *args, **kw):
         """replaces: os.makedirs()"""
         self.add_usage(path, 'update .makedirs to use Path.mkdir()')
         os.makedirs(str(path), *args, **kw)
 
-    def remove(self, path: Union[str, Path]) -> Any:
+    def remove(self, path):
         """replaces: os.remove()"""
-        self.add_usage(path, 'update .remove to use Path.remove() or os.remove(str(Path))')
+        self.add_usage(path, 'update .chdir to use Path.chdir() or os.chdir(str(Path))')
         if isinstance(path, Path):
-            return path.remove()  # type: ignore
+            return path.remove()
         else:
             return os.remove(path)
 
-    def listdir(self, path: Union[str, Path] = '.') -> Any:
+    def listdir(self, path='.'):
         """replaces: os.listdir()"""
         return os.listdir(str(path))
 
-    def glob(self, pattern: str) -> Union[List[str], List[Path]]:
+    def glob(self, pattern):
         """replaces: glob.glob()"""
         import glob
 
-        self.add_usage(pattern, 'update .glob to use Path.glob()')
+        self.add_usage(path, 'update .glob to use Path.glob()')
         if isinstance(pattern, Path):
             return [Path(fn) for fn in glob.glob(str(pattern))]
         else:
             return glob.glob(pattern)
 
     # shutil.rmtree
-    def rmtree(self, path: Union[str, Path]) -> Any:
+    def rmtree(self, path):
         """replaces: shutil.rmtree()"""
         import shutil
 
         self.add_usage(path, 'update .rmtree to use Path.rmtree() or shutil.rmtree(str(Path))')
         if isinstance(path, Path):
-            return path.rmtree()  # type: ignore
+            return path.rmtree()
         else:
             return shutil.rmtree(path)
 
     # shutil.copy
-    def copy(self, src: Union[str, Path], dst: Union[str, Path]) -> Any:
+    def copy(self, src, dst):
         """replaces: shutil.copy()"""
         import shutil
 
@@ -369,12 +364,12 @@ class PathLibConversionHelper(object):
             src, 'update .copy to use Path.copy() or shutil.copy(str(src), str(dst))'
         )
         if isinstance(src, Path):
-            return src.copy(dst)  # type: ignore
+            return src.copy(dst)
         else:
             return shutil.copy(str(str), str(dst))
 
     # built-in open
-    def open(self, path: Union[str, Path], mode: str = 'r', buffering: int = 1) -> Any:
+    def open(self, path, mode='r', buffering=1):
         """replaces: built-in open()"""
         self.add_usage(path, 'update .open to use Path.open()')
         if isinstance(path, Path):
@@ -382,13 +377,7 @@ class PathLibConversionHelper(object):
         else:
             return open(path, mode, buffering)
 
-    def mkstemp(
-        self,
-        suffix: Any = "",
-        prefix: Optional[str] = None,
-        dir: Optional[Union[str, Path]] = None,
-        text: bool = False,
-    ) -> Any:
+    def mkstemp(self, suffix="", prefix=None, dir=None, text=False):
         """replaces: tempfile.mkstemp()"""
         import tempfile
 
